#!/usr/bin/env python
# -*- coding: ASCII -*-

"""

:Author: Martin Kircher
:Contact: mkircher@uw.edu
:Date: *09.05.2012

"""

import os,sys
from collections import defaultdict
from optparse import OptionParser
import traceback

parser = OptionParser()
parser.add_option("-c","--chroms", dest="chromosomes", help="Chromosomes considered (def '1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22')",default='1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22')
parser.add_option("-p","--params_folder",dest="params_folder",help="Folder with EPO parameter files (def Ensembl_Compara_v66_EPO_6primate/)",default="Ensembl_Compara_v66_EPO_6primate/")
parser.add_option("-o", "--outfile", dest="outfile", help="Name of output file (def STDOUT)",default=None)
(options, args) = parser.parse_args()

if not os.path.isdir(options.params_folder):
  sys.stderr.write("Invalid path for EPO parameter files.\n")
  sys.exit()

if options.chromosomes != '':
  filenames = map(lambda x: options.params_folder+"/%s.log"%x,options.chromosomes.split(','))
else:
  filenames = filter(lambda x: x.endswith(".log"), os.path.listdir(options.params_folder))

totalrefA = 0
totalrefC = 0
totalrefG = 0
totalrefT = 0

mut,total = 0,0
nAC,nAG,nAT,nCA,nCG,nCT,nGA,nGC,nGT,nTA,nTC,nTG = 0,0,0,0,0,0,0,0,0,0,0,0
totalCpG,mutCpG = 0,0
mCA,mCG,mCT,mGA,mGC,mGT = 0,0,0,0,0,0

insertionsizes,deletionsizes = defaultdict(int),defaultdict(int)

intervals = defaultdict(list)
interval_vals = {}

control_mut,control_mutCpG,control_total = 0,0,0
try:
  for filename in filenames:
    infile = open(filename)
    line = infile.readline()
    while line != "":
      if line == "#A\tC\tG\tT\tCpGs\n":
        line = infile.readline()
        A,C,G,T,cCpG = map(int,line.split())
        totalrefA += A
        totalrefC += C
        totalrefG += G
        totalrefT += T
        totalCpG += cCpG
      elif line == "#y\tN\tAC\tAG\tAT\tCA\tCG\tCT\tGA\tGC\tGT\tTA\tTC\tTG\n":
        line = infile.readline()
        cmut,ctotal,cAC,cAG,cAT,cCA,cCG,cCT,cGA,cGC,cGT,cTA,cTC,cTG = map(int,line.split())
        mut+=cmut
        total+=ctotal
        nAC+=cAC
        nAG+=cAG
        nAT+=cAT
        nCA+=cCA
        nCG+=cCG
        nCT+=cCT
        nGA+=cGA
        nGC+=cGC
        nGT+=cGT
        nTA+=cTA
        nTC+=cTC
        nTG+=cTG
      elif line == "#yCpG\tNCpG\tCA\tCG\tCT\tGA\tGC\tGT\n":
        line = infile.readline()
        cmutCpG,ctotalCpG,cCA,cCG,cCT,cGA,cGC,cGT = map(int,line.split())
        mutCpG+=cmutCpG
        totalCpG+=ctotalCpG
        mCA+=cCA
        mCG+=cCG
        mCT+=cCT
        mGA+=cGA
        mGC+=cGC
        mGT+=cGT
      elif line == "##INSERTIONS\n":
        line = infile.readline() # HEADER
        line = infile.readline()
        while line != "" and not line.startswith('#'):
          fields = line.split()
          if len(fields) == 2:
            length,count=map(int,fields[:2])
            insertionsizes[length]+=count
          line = infile.readline()
      elif line == "##DELETIONS\n":
        line = infile.readline() # HEADER
        line = infile.readline()
        while line != "" and not line.startswith('#'):
          fields = line.split()
          if len(fields) == 2:
            length,count=map(int,fields[:2])
            deletionsizes[length]+=count
          line = infile.readline()
      else:
        fields = line.split()
        if len(fields) == 17:
          chrom = fields[0]
          window_start,window_end,cmut,ctotal,cmutCpG,ctotalCpG,cA,cC,cG,cT = map(int,fields[1:11])
          control_mut+=cmut
          control_mutCpG+=cmutCpG
          control_total+=ctotal
          intervals[chrom].append((window_start, window_end+1))
          interval_vals[(chrom,window_start, window_end+1)]=(cmut,ctotal,cmutCpG,ctotalCpG,cA,cC,cG,cT)
        line = infile.readline()
    infile.close()
except:
  exc_type, exc_value, exc_traceback = sys.exc_info()
  sys.stderr.write("%s\n"%str(exc_value))
  traceback.print_tb(exc_traceback)
  sys.stderr.write('Script terminated early. Printing current values.\n')

sys.stderr.write("Chromosomes considered: %s\n"%len(intervals))
sys.stderr.write("Number of regions: %d\n"%len(interval_vals))

sum_obs = float(totalrefA+totalrefC+totalrefG+totalrefT)
total = float(total)
totalCpG = float(totalCpG)
sys.stderr.write("Total aligned positions: %d (Exp: %d)\n"%(sum_obs,control_total))
sys.stderr.write("Total mutations: %d/%d (Exp: %d/%d); Rate: %.8f/%.8f\n"%(mut,mutCpG,control_mut,control_mutCpG,mut/total,mutCpG/totalCpG))
gmut = mut/total
gmutCpG = mutCpG/totalCpG

gmut_ = mut/sum_obs
gmutCpG_ = mutCpG/sum_obs
sys.stderr.write("Abs. mutation rate non-CpG: %.8f\tAbs. mutation rate CpGs: %.8f\n"%(gmut_,gmutCpG_))

# RATES DETERMINED FROM HUMAN-CHIMP ANCESTOR (CORRECTED FOR BASE COMPOSITION)

totalrefA = float(totalrefA)
totalrefC = float(totalrefC)
totalrefG = float(totalrefG)
totalrefT = float(totalrefT)

##SUBSTITUTION MATRIX NON-CpG
GTR_nonCpG = {'A':[             0 , nAC/totalrefA , nAG/totalrefA , nAT/totalrefA ],
              'C':[ nCA/totalrefC ,             0 , nCG/totalrefC , nCT/totalrefC ],
              'G':[ nGA/totalrefG , nGC/totalrefG ,             0 , nGT/totalrefG ],
              'T':[ nTA/totalrefT , nTC/totalrefT , nTG/totalrefT ,             0 ] }

ffactor = map(lambda x: 1.0/sum(x),GTR_nonCpG.values())
print "nonCpG",GTR_nonCpG.keys()
for ki,key in enumerate(GTR_nonCpG.keys()):
  print key, GTR_nonCpG[key]
  for i in range(4):
    GTR_nonCpG[key][i] = GTR_nonCpG[key][i] * ffactor[ki]
  print key, GTR_nonCpG[key]

##SUBSTITUTION MATRIX CpG
mtotalrefC = (totalCpG-(mCA+mCG+mCT+mGA+mGC+mGT))/2.0+mCA+mCG+mCT
mtotalrefG = (totalCpG-(mCA+mCG+mCT+mGA+mGC+mGT))/2.0+mGA+mGC+mGT

GTR_CpG = {   'C':[ mCA/mtotalrefC ,              0 , mCG/mtotalrefC , mCT/mtotalrefC ],
              'G':[ mGA/mtotalrefG , mGC/mtotalrefG ,              0 , mGT/mtotalrefG ] }

ffactor = map(lambda x: 1.0/sum(x),GTR_CpG.values())
print "CpG",GTR_CpG.keys()
for ki,key in enumerate(GTR_CpG.keys()):
  print key, GTR_CpG[key]
  for i in range(4):
    GTR_CpG[key][i] = GTR_CpG[key][i] * ffactor[ki]
  print key, GTR_CpG[key]

# INITIATE INSERT NORMALIZED LIKELIHOODS
inserts = []
tinserts = float(sum(insertionsizes.values()))
sys.stderr.write("TOTAL INSERTS FROM EPO:\t%d\n"%tinserts)
to_sort = insertionsizes.keys()
to_sort.sort()
sys.stderr.write("Length\tCount\tnFreq\n")
for key in to_sort:
  value = insertionsizes[key]/tinserts
  #if key <= 10: sys.stderr.write("%d\t%d\t%.8f\n"%(key, insertionsizes[key], value))
  inserts.append((key,value))
del insertionsizes
ginserts = tinserts/sum_obs
sys.stderr.write("INSERTIONS: gfreq=%.8f, %s...\n"%(ginserts,str(inserts)[:120]))

# INITIATE DELETION NORMALIZED LIKELIHOODS
deletions = []
tdeletions = float(sum(deletionsizes.values()))
sys.stderr.write("TOTAL DELETIONS FROM EPO:\t%d\n"%tdeletions)
to_sort = deletionsizes.keys()
to_sort.sort()
sys.stderr.write("Length\tCount\tnFreq\n")
for key in to_sort:
  value = deletionsizes[key]/tdeletions
  #if key <= 10: sys.stderr.write("%d\t%d\t%.8f\n"%(key, deletionsizes[key], value))
  deletions.append((key,value))
del deletionsizes
gdeletions = tdeletions/sum_obs
sys.stderr.write("DELETIONS: gfreq=%.8f, %s...\n"%(gdeletions,str(deletions)[:120]))
